@@ -1,7 +1,7 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use Clone;
 use HTML::Element;
@@ -43,65 +43,77 @@ sub reverse   {
 sub process {
     my ($self,$data,$args) = _args( @_ );
 
-    $data = Clone::clone( $self->{data} ) if $self->{is_cached};
+    # return if cached
+    if (ref($self) and $self->{is_cached}) {
+        return wantarray ? ( data => $self->{data}, %$args ) : $data;
+    }
 
     my $max_cols = scalar @{ $data->[0] };
 
-    for my $row (@$data) {
+    # assign each cell as a td first
+    my $first = 1;
+    for my $row (@{$data}) {
+
+        # pad and truncate
         push @$row, undef for 1 .. ($max_cols - scalar @$row);
-        #truncate rows that are too long
         $row = [ @$row[0 .. $max_cols - 1] ] if scalar( @$row ) > $max_cols;
 
-        # white space transliteration
         for my $i (0 .. $#$row) {
-            #TODO: this needs to be configurable
-            do{ no warnings; $row->[$i] =~ s/^\s*$/&nbsp;/g };
-            $row->[$i] =~ s/\n/<br \/>/g;
+            $row->[$i] = ($first and !($args->{headless} or $args->{matrix}))
+                ? _element( th => _scrub( $row->[$i] ), $args->{th} )
+                : _element( td => _scrub( $row->[$i] ), $args->{td} )
+            ;
         }
+
+        $first = 0;
     }
 
-    unless ( $args->{headless} or $args->{matrix} or ref($data->[0][0]) ) {
-        #TODO: make objects here, assign class attrs here
-        $data->[0] = [ map [$_], @{ $data->[0] } ];
+    if (ref($self) and !$self->{is_cached} and $args->{cache}) {
+        $self->{data} = $data;
+        $self->{is_cached} = 1;
     }
 
-    if ($args->{headless}) {
-        shift @$data;
-    }
+    shift @$data if $args->{headless};
 
     return wantarray ? ( data => $data, %$args ) : $data;
 }
 
-sub _make_table {
-    my %args = @_;
-    $args{$_} ||= {} for qw( table tr th td );
-
-    my $indent  = $args{indent};
-    my $no_th   = $args{matrix};
-    my $encodes = exists $args{encodes} ? $args{encodes} : '';
-
-    my $table = HTML::Element->new_from_lol(
-        [table => $args{table},
-            map [tr => $args{tr},
-                map ref($_)
-                    ? $no_th
-                        ? [ td => $args{td}, @$_ ]
-                        : [ th => $args{th}, @$_ ]
-                    : [ td => $args{td}, $_ ], @$_
-            ], @{ $args{data} }
-        ],
-    );
-
-    chomp( my $html = $table->as_HTML( $encodes, $indent ) );
-    return $html;
-}
-
 sub new {
     my $class = shift;
     my %attrs = ref($_[0]) eq 'HASH' ? %{+shift} : @_;
     return bless { %attrs }, $class;
 }
 
+sub _make_table {
+    my %args = @_;
+    $args{$_} ||= {} for qw( table tr );
+
+    my $encodes = exists $args{encodes} ? $args{encodes} : '';
+
+    my $table = HTML::Element->new_from_lol(
+        [table => $args{table},
+            map [tr => $args{tr}, @$_ ], @{ $args{data} }
+        ],
+    );
+
+    chomp( my $html = $table->as_HTML( $encodes, $args{indent} ) );
+    return $html;
+}
+
+sub _scrub {
+    my $value = shift;
+    do{ no warnings; $value =~ s/^\s*$/&nbsp;/g };
+    $value =~ s/\n/<br \/>/g;
+    return $value;
+}
+
+sub _element {
+    my ($tag, $content, $attr) = @_;
+    my $e = HTML::Element->new( $tag, %{ $attr || {} } );
+    $e->push_content( $content );
+    return $e;
+}
+
 sub _args {
     my ($self,$data,$args);
     $self = shift if UNIVERSAL::isa( $_[0], __PACKAGE__ );
@@ -119,12 +131,16 @@ sub _args {
     }
 
     if ($self) {
-        $args = { %$self, %{ $args || {} } };
+        $args = { %{ ref($self) ? $self : {} }, %{ $args || {} } };
         delete $args->{data};
     }
 
+    if (ref($self) and $self->{is_cached}) {
+        return ( $self, $self->{data}, $args );
+    }
+
     unless (exists $args->{file}) {
-        if ($self and !$data) {
+        if (ref($self) and !$data) {
             $data = $self->{data};
         }
         $data = [ $data ] unless ref($data);
@@ -144,11 +160,6 @@ sub _args {
         }
     }
 
-    if ($self and !$self->{is_cached} and delete $args->{cache}) {
-        $self->{data} = $data;
-        $self->{is_cached} = 1;
-    }
-
     return ( $self, Clone::clone($data), $args );
 }
 
@@ -327,8 +338,7 @@ encoded as HTML entites (see L<HTML::Element>::as_HTML).
 
 =item * cache => 0 or 1
 
-Preserve data after it has been processed. Only makes
-sense when used as class method on an object.
+Preserve data after it has been processed (and loaded).
 
 =item * matrix => 0 or 1
 
@@ -396,6 +406,21 @@ suggest action. This version may or may not reflect that. ;)
 
 =back
 
+=head1 SEE ALSO
+
+=over 4
+
+=item * L<DBIx::HTML>
+
+Uses this module (Spreadsheet::HTML) to format SQL query results.
+
+=item * L<DBIx::XHTML_Table>
+
+The original since 2001. Can handle advanced grouping, individual cell
+value contol, rotating attributes and totals/subtotals.
+
+=back
+
 =head1 BUGS
 
 Please report any bugs or feature requests to either
